@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Nick Schermer <nick@xfce.org>
+ * Copyright (C) 2009-2010 Nick Schermer <nick@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -363,6 +363,10 @@ G_BEGIN_DECLS
             gtk_widget_queue_draw (plug); \
             break; \
             \
+          case PROVIDER_PROP_TYPE_ACTION_ASK_REMOVE: \
+            xfce_panel_plugin_provider_ask_remove (provider); \
+            break; \
+            \
           default: \
             g_warning ("Received unknow client event %u", type); \
             break; \
@@ -416,6 +420,8 @@ G_BEGIN_DECLS
   _xpp_realize (XfcePanelPlugin *xpp, \
                 GtkPlug         *plug) \
   { \
+    GtkWidget *ebox; \
+    \
     g_return_if_fail (XFCE_IS_PANEL_PLUGIN (xpp)); \
     g_return_if_fail (GTK_IS_PLUG (plug)); \
     g_return_if_fail (GTK_WIDGET_REALIZED (plug)); \
@@ -431,6 +437,10 @@ G_BEGIN_DECLS
         G_CALLBACK (_xpp_provider_signal), plug); \
     \
     ((XfcePanelPluginFunc) construct_func) (xpp); \
+    \
+    ebox = gtk_bin_get_child (GTK_BIN (xpp)); \
+    if (ebox != NULL && GTK_IS_EVENT_BOX (ebox)) \
+      gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE); \
   } \
   \
   static gboolean \
@@ -579,10 +589,7 @@ G_BEGIN_DECLS
     gtk_widget_set_app_paintable (plug, TRUE); \
     \
     screen = gtk_widget_get_screen (plug); \
-    \
     colormap = gdk_screen_get_rgba_colormap (screen); \
-    if (colormap == NULL) \
-      colormap = gdk_screen_get_rgb_colormap (screen); \
     if (colormap != NULL) \
       gtk_widget_set_colormap (plug, colormap); \
     \
