@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2009 Nick Schermer <nick@xfce.org>
+ * Copyright (C) 2008-2010 Nick Schermer <nick@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,35 +38,57 @@
 #include <panel/panel-item-dialog.h>
 #include <panel/panel-preferences-dialog.h>
 #include <panel/panel-preferences-dialog-ui.h>
+#include <panel/panel-plugin-external.h>
 
 #define PREFERENCES_HELP_URL "http://www.xfce.org"
 
 
 
-static void panel_preferences_dialog_finalize (GObject *object);
-static void panel_preferences_dialog_response (GtkWidget *window, gint response_id, PanelPreferencesDialog *dialog);
-
-
-static void panel_preferences_dialog_bindings_unbind (PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_bindings_add (PanelPreferencesDialog *dialog, const gchar *property1, const gchar *property2);
-static void panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog);
-
-static void panel_preferences_dialog_output_changed (GtkComboBox *combobox, PanelPreferencesDialog *dialog);
-
-static void panel_preferences_dialog_panel_combobox_changed (GtkComboBox *combobox, PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_panel_combobox_rebuild (PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_panel_add (GtkWidget *widget, PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_panel_remove (GtkWidget *widget, PanelPreferencesDialog *dialog);
-
-
-static XfcePanelPluginProvider *panel_preferences_dialog_item_get_selected (PanelPreferencesDialog *dialog, GtkTreeIter *return_iter);
-static void panel_preferences_dialog_item_store_rebuild (GtkWidget *itembar, PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_item_move (GtkWidget *button, PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_item_remove (GtkWidget *button, PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_item_add (GtkWidget *button, PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_item_properties (GtkWidget *button, PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_item_about (GtkWidget *button, PanelPreferencesDialog *dialog);
-static void panel_preferences_dialog_item_selection_changed (GtkTreeSelection *selection, PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_finalize               (GObject                *object);
+static void                     panel_preferences_dialog_response               (GtkWidget              *window,
+                                                                                 gint                    response_id,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_bindings_unbind        (PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_bindings_add           (PanelPreferencesDialog *dialog,
+                                                                                 const gchar            *property1,
+                                                                                 const gchar            *property2);
+static void                     panel_preferences_dialog_bindings_update        (PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_output_changed         (GtkComboBox            *combobox,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_bg_style_changed       (PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_bg_image_file_set      (GtkFileChooserButton   *button,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_bg_image_notified      (PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_panel_combobox_changed (GtkComboBox            *combobox,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_panel_combobox_rebuild (PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_panel_add              (GtkWidget              *widget,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_panel_remove           (GtkWidget              *widget,
+                                                                                 PanelPreferencesDialog *dialog);
+static XfcePanelPluginProvider *panel_preferences_dialog_item_get_selected      (PanelPreferencesDialog *dialog,
+                                                                                 GtkTreeIter            *return_iter);
+static void                     panel_preferences_dialog_item_store_rebuild     (GtkWidget              *itembar,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_item_move              (GtkWidget              *button,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_item_remove            (GtkWidget              *button,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_item_add               (GtkWidget              *button,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_item_properties        (GtkWidget              *button,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_item_about             (GtkWidget              *button,
+                                                                                 PanelPreferencesDialog *dialog);
+static gboolean                 panel_preferences_dialog_treeview_clicked       (GtkTreeView            *treeview,
+                                                                                 GdkEventButton         *event,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_item_row_changed       (GtkTreeModel           *model,
+                                                                                 GtkTreePath            *path,
+                                                                                 GtkTreeIter            *iter,
+                                                                                 PanelPreferencesDialog *dialog);
+static void                     panel_preferences_dialog_item_selection_changed (GtkTreeSelection       *selection,
+                                                                                 PanelPreferencesDialog *dialog);
 
 
 
@@ -108,6 +130,9 @@ struct _PanelPreferencesDialog
   /* changed signal for the active panel's itembar */
   gulong            items_changed_handler_id;
 
+  /* background image watch */
+  gulong            bg_image_notify_handler_id;
+
   /* changed signal for the output selector */
   gulong            output_changed_handler_id;
 };
@@ -167,6 +192,22 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   connect_signal ("panel-remove", "clicked", panel_preferences_dialog_panel_remove);
   connect_signal ("panel-combobox", "changed", panel_preferences_dialog_panel_combobox_changed);
 
+  /* style tab */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-style");
+  panel_return_if_fail (G_IS_OBJECT (object));
+  g_signal_connect_swapped (G_OBJECT (object), "changed",
+      G_CALLBACK (panel_preferences_dialog_bg_style_changed), dialog);
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "composited");
+  panel_return_if_fail (G_IS_OBJECT (object));
+  g_signal_connect_swapped (G_OBJECT (object), "notify::visible",
+      G_CALLBACK (panel_preferences_dialog_bg_style_changed), dialog);
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-image");
+  panel_return_if_fail (GTK_IS_FILE_CHOOSER_BUTTON (object));
+  g_signal_connect (G_OBJECT (object), "file-set",
+    G_CALLBACK (panel_preferences_dialog_bg_image_file_set), dialog);
+
   /* items treeview and buttons */
   connect_signal ("item-up", "clicked", panel_preferences_dialog_item_move);
   connect_signal ("item-down", "clicked", panel_preferences_dialog_item_move);
@@ -187,6 +228,12 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   panel_return_if_fail (GTK_IS_WIDGET (treeview));
   gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (dialog->store));
   gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (treeview), ITEM_COLUMN_TOOLTIP);
+  g_signal_connect (G_OBJECT (treeview), "button-press-event",
+      G_CALLBACK (panel_preferences_dialog_treeview_clicked), dialog);
+
+  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), TRUE);
+  g_signal_connect (G_OBJECT (dialog->store), "row-changed",
+      G_CALLBACK (panel_preferences_dialog_item_row_changed), dialog);
 
   /* setup tree selection */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
@@ -205,7 +252,7 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   renderer = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
-  gtk_tree_view_column_set_attributes (column, renderer, "text",
+  gtk_tree_view_column_set_attributes (column, renderer, "markup",
                                        ITEM_COLUMN_DISPLAY_NAME, NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
@@ -232,12 +279,24 @@ panel_preferences_dialog_finalize (GObject *object)
   PanelPreferencesDialog *dialog = PANEL_PREFERENCES_DIALOG (object);
   GtkWidget              *itembar;
 
-  /* disconnect changed signal */
-  if (dialog->active != NULL && dialog->items_changed_handler_id != 0)
+  /* free bindings list */
+  g_slist_free (dialog->bindings);
+
+  if (dialog->active != NULL)
     {
-      itembar = gtk_bin_get_child (GTK_BIN (dialog->active));
-      g_signal_handler_disconnect (G_OBJECT (itembar),
-          dialog->items_changed_handler_id);
+      if (dialog->items_changed_handler_id != 0)
+        {
+          /* disconnect changed signal */
+          itembar = gtk_bin_get_child (GTK_BIN (dialog->active));
+          g_signal_handler_disconnect (G_OBJECT (itembar),
+              dialog->items_changed_handler_id);
+        }
+
+      if (dialog->bg_image_notify_handler_id != 0)
+        {
+          g_signal_handler_disconnect (G_OBJECT (dialog->active),
+              dialog->bg_image_notify_handler_id);
+        }
     }
 
   /* deselect all windows */
@@ -278,7 +337,7 @@ panel_preferences_dialog_bindings_unbind (PanelPreferencesDialog *dialog)
 {
   GSList *li;
 
-  if (dialog->bindings)
+  if (dialog->bindings != NULL)
     {
       /* remove all bindings */
       for (li = dialog->bindings; li != NULL; li = li->next)
@@ -287,6 +346,18 @@ panel_preferences_dialog_bindings_unbind (PanelPreferencesDialog *dialog)
       g_slist_free (dialog->bindings);
       dialog->bindings = NULL;
     }
+
+  /* disconnect image watch */
+  if (dialog->bg_image_notify_handler_id != 0)
+    {
+      if (dialog->active != NULL)
+        {
+          g_signal_handler_disconnect (G_OBJECT (dialog->active),
+              dialog->bg_image_notify_handler_id);
+        }
+
+      dialog->bg_image_notify_handler_id = 0;
+    }
 }
 
 
@@ -326,9 +397,6 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   gchar       *name, *title;
   gboolean     span_monitors_sensitive = FALSE;
 
-  /* remove all the active bindings */
-  panel_preferences_dialog_bindings_unbind (dialog);
-
   /* leave when there is no active panel */
   panel_return_if_fail (G_IS_OBJECT (dialog->active));
   if (dialog->active == NULL)
@@ -346,6 +414,13 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   panel_preferences_dialog_bindings_add (dialog, "enter-opacity", "value");
   panel_preferences_dialog_bindings_add (dialog, "leave-opacity", "value");
   panel_preferences_dialog_bindings_add (dialog, "composited", "visible");
+  panel_preferences_dialog_bindings_add (dialog, "background-style", "active");
+  panel_preferences_dialog_bindings_add (dialog, "background-color", "color");
+
+  /* watch image changes from the panel */
+  dialog->bg_image_notify_handler_id = g_signal_connect_swapped (G_OBJECT (dialog->active),
+      "notify::background-image", G_CALLBACK (panel_preferences_dialog_bg_image_notified), dialog);
+  panel_preferences_dialog_bg_image_notified (dialog);
 
   /* get run mode of the driver (multiple screens or randr) */
   display = gtk_widget_get_display (GTK_WIDGET (dialog->active));
@@ -507,6 +582,84 @@ panel_preferences_dialog_output_changed (GtkComboBox            *combobox,
 
 
 
+static void
+panel_preferences_dialog_bg_style_changed (PanelPreferencesDialog *dialog)
+{
+  gint      active;
+  GObject  *object;
+  gboolean  composited;
+
+  panel_return_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog));
+  panel_return_if_fail (PANEL_WINDOW (dialog->active));
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "background-style");
+  panel_return_if_fail (GTK_IS_COMBO_BOX (object));
+  active = gtk_combo_box_get_active (GTK_COMBO_BOX (object));
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "bg-alpha-box");
+  panel_return_if_fail (GTK_IS_WIDGET (object));
+  g_object_get (G_OBJECT (dialog->active), "composited", &composited, NULL);
+  g_object_set (G_OBJECT (object), "visible", composited && active < 2, NULL);
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "bg-color-box");
+  panel_return_if_fail (GTK_IS_WIDGET (object));
+  g_object_set (G_OBJECT (object), "visible", active == 1, NULL);
+
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "bg-image-box");
+  panel_return_if_fail (GTK_IS_WIDGET (object));
+  g_object_set (G_OBJECT (object), "visible", active == 2, NULL);
+}
+
+
+
+static void
+panel_preferences_dialog_bg_image_file_set (GtkFileChooserButton   *button,
+                                            PanelPreferencesDialog *dialog)
+{
+  gchar *filename;
+
+  panel_return_if_fail (GTK_IS_FILE_CHOOSER_BUTTON (button));
+  panel_return_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog));
+  panel_return_if_fail (PANEL_IS_WINDOW (dialog->active));
+
+  g_signal_handler_block (G_OBJECT (dialog->active),
+      dialog->bg_image_notify_handler_id);
+
+  filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (button));
+  g_object_set (G_OBJECT (dialog->active), "background-image", filename, NULL);
+  g_free (filename);
+
+  g_signal_handler_unblock (G_OBJECT (dialog->active),
+      dialog->bg_image_notify_handler_id);
+}
+
+
+
+static void
+panel_preferences_dialog_bg_image_notified (PanelPreferencesDialog *dialog)
+{
+  gchar   *filename;
+  GObject *button;
+
+  panel_return_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog));
+  panel_return_if_fail (PANEL_IS_WINDOW (dialog->active));
+
+  button = gtk_builder_get_object (GTK_BUILDER (dialog), "background-image");
+  panel_return_if_fail (GTK_IS_FILE_CHOOSER_BUTTON (button));
+
+  g_signal_handlers_block_by_func (G_OBJECT (button),
+      G_CALLBACK (panel_preferences_dialog_bg_image_file_set), dialog);
+
+  g_object_get (G_OBJECT (dialog->active), "background-image", &filename, NULL);
+  gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (button), filename != NULL ? filename : "");
+  g_free (filename);
+
+  g_signal_handlers_unblock_by_func (G_OBJECT (button),
+      G_CALLBACK (panel_preferences_dialog_bg_image_file_set), dialog);
+}
+
+
+
 static void
 panel_preferences_dialog_panel_sensitive (PanelPreferencesDialog *dialog)
 {
@@ -558,6 +711,9 @@ panel_preferences_dialog_panel_combobox_changed (GtkComboBox            *combobo
       g_signal_handler_disconnect (G_OBJECT (itembar), dialog->items_changed_handler_id);
     }
 
+  /* remove all the active bindings */
+  panel_preferences_dialog_bindings_unbind (dialog);
+
   /* set the selected window */
   nth = gtk_combo_box_get_active (combobox);
   dialog->active = panel_application_get_nth_window (dialog->application, nth);
@@ -731,7 +887,7 @@ panel_preferences_dialog_item_store_rebuild (GtkWidget              *itembar,
   GList       *items, *li;
   guint        i;
   PanelModule *module;
-  gchar       *tooltip;
+  gchar       *tooltip, *display_name;
 
   panel_return_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog));
   panel_return_if_fail (GTK_IS_LIST_STORE (dialog->store));
@@ -739,31 +895,59 @@ panel_preferences_dialog_item_store_rebuild (GtkWidget              *itembar,
 
   gtk_list_store_clear (dialog->store);
 
+  g_signal_handlers_block_by_func (G_OBJECT (dialog->store),
+      G_CALLBACK (panel_preferences_dialog_item_row_changed), dialog);
+
   /* add items to the store */
   items = gtk_container_get_children (GTK_CONTAINER (itembar));
   for (li = items, i = 0; li != NULL; li = li->next, i++)
     {
-      /* I18N: tooltip in preferences dialog when hovering an item in the list */
-      tooltip = g_strdup_printf (_("Internal name: %s-%d"),
-                                 xfce_panel_plugin_provider_get_name (li->data),
-                                 xfce_panel_plugin_provider_get_unique_id (li->data));
-
       /* get the panel module from the plugin */
       module = panel_module_get_from_plugin_provider (li->data);
 
+      if (PANEL_IS_PLUGIN_EXTERNAL (li->data))
+        {
+          /* I18N: append (external) in the preferences dialog if the plugin
+           * runs external */
+          display_name = g_strdup_printf (_("%s <span color=\"grey\" size=\"small\">(external)</span>"),
+                                          panel_module_get_display_name (module));
+
+          /* I18N: tooltip in preferences dialog when hovering an item in the list
+           * for external plugins */
+          tooltip = g_strdup_printf (_("Internal name: %s-%d\n"
+                                       "PID: %d"),
+                                     xfce_panel_plugin_provider_get_name (li->data),
+                                     xfce_panel_plugin_provider_get_unique_id (li->data),
+                                     panel_plugin_external_get_pid (PANEL_PLUGIN_EXTERNAL (li->data)));
+        }
+      else
+        {
+          display_name = g_strdup (panel_module_get_display_name (module));
+
+          /* I18N: tooltip in preferences dialog when hovering an item in the list
+           * for internal plugins */
+          tooltip = g_strdup_printf (_("Internal name: %s-%d"),
+                                     xfce_panel_plugin_provider_get_name (li->data),
+                                     xfce_panel_plugin_provider_get_unique_id (li->data));
+        }
+
       gtk_list_store_insert_with_values (dialog->store, NULL, i,
                                          ITEM_COLUMN_ICON_NAME,
                                          panel_module_get_icon_name (module),
                                          ITEM_COLUMN_DISPLAY_NAME,
-                                         panel_module_get_display_name (module),
+                                         display_name,
                                          ITEM_COLUMN_TOOLTIP,
                                          tooltip,
                                          ITEM_COLUMN_PROVIDER, li->data, -1);
 
       g_free (tooltip);
+      g_free (display_name);
     }
 
   g_list_free (items);
+
+  g_signal_handlers_unblock_by_func (G_OBJECT (dialog->store),
+      G_CALLBACK (panel_preferences_dialog_item_row_changed), dialog);
 }
 
 
@@ -924,6 +1108,72 @@ panel_preferences_dialog_item_about (GtkWidget              *button,
 
 
 
+static gboolean
+panel_preferences_dialog_treeview_clicked (GtkTreeView            *treeview,
+                                           GdkEventButton         *event,
+                                           PanelPreferencesDialog *dialog)
+{
+  gint x, y;
+
+  panel_return_val_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog), FALSE);
+  panel_return_val_if_fail (GTK_IS_TREE_VIEW (treeview), FALSE);
+
+  gtk_tree_view_convert_widget_to_bin_window_coords (treeview,
+                                                     event->x, event->y,
+                                                     &x, &y);
+
+  /* open preferences on double-click on a row */
+  if (event->type == GDK_2BUTTON_PRESS
+      && event->button == 1
+      && gtk_tree_view_get_path_at_pos (treeview, x, y, NULL, NULL, NULL, NULL))
+    {
+      panel_preferences_dialog_item_properties (NULL, dialog);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+
+
+static void
+panel_preferences_dialog_item_row_changed (GtkTreeModel           *model,
+                                           GtkTreePath            *path,
+                                           GtkTreeIter            *iter,
+                                           PanelPreferencesDialog *dialog)
+{
+  XfcePanelPluginProvider *provider = NULL;
+  gint                     position;
+  GtkWidget               *itembar;
+  gint                     store_position;
+
+  panel_return_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog));
+  panel_return_if_fail (GTK_TREE_MODEL (dialog->store) == model);
+  panel_return_if_fail (PANEL_IS_WINDOW (dialog->active));
+
+  /* get the changed row */
+  gtk_tree_model_get (model, iter, ITEM_COLUMN_PROVIDER, &provider, -1);
+  panel_return_if_fail (XFCE_IS_PANEL_PLUGIN_PROVIDER (provider));
+  store_position = gtk_tree_path_get_indices (path)[0];
+
+  /* actual position on the panel */
+  itembar = gtk_bin_get_child (GTK_BIN (dialog->active));
+  position = panel_itembar_get_child_index (PANEL_ITEMBAR (itembar),
+                                            GTK_WIDGET (provider));
+
+  /* correct position in the list */
+  if (position < store_position)
+    store_position--;
+
+  /* move the item on the panel */
+  if (position != store_position)
+    panel_itembar_reorder_child (PANEL_ITEMBAR (itembar),
+                                 GTK_WIDGET (provider),
+                                 store_position);
+}
+
+
+
 static void
 panel_preferences_dialog_item_selection_changed (GtkTreeSelection       *selection,
                                                  PanelPreferencesDialog *dialog)
